var changed = false;

window.addEventListener("beforeunload", function (e) {
	if( changed ) {
		need_focus = true;
		
		// Cancel the event as stated by the standard.
		e.preventDefault();

		// The browser will replace this text with its own
		var message = "There are unsaved changes. Really leave ?",  e = e || window.event;
		// For IE and Firefox
		if (e) {
			e.returnValue = message;
		}

		// For Safari
		return message;
	}
});
	
function goto_settings(e) {
	var dst = $(this).data("dest");
	window.location.replace(dst);
}

function get_depth(instance,id,current_max)
{
	var new_max = current_max;
	var node = instance.get_node(id);
	var path = instance.get_path(node);
	if (typeof path == "object") {
		var path_len = path.length;
		if (path_len > new_max)
			new_max = path_len;
	}
	
	for (var i=0; i<node.children.length; i++) {
		var m = get_depth(instance,node.children[i],new_max);
		if (m > new_max)
			new_max = m;
	}
	
	return new_max;
}

function action_func(action) {
	/*
		action function will be invoked with an object containing the following keys:
		item - the contextmenu item definition, 
		reference - the DOM node that was used (the tree node), 
		element - the contextmenu DOM element, 
		position - an object with x/y properties indicating the position of the menu.
	*/
	
	var lab = action.item.label;
	if(lab != "Configure")
		changed = true;
	
	var target = action.reference;
	var inst = $.jstree.reference(target);
	var target_node = inst.get_node(target);
	var index = target_node.id;
	var folder = target_node.original.folder;
	
	if (lab == "Delete") {
		var r = true;
		if (inst.is_parent(target_node)) {
			var startTime = new Date().getTime();
			var r = confirm("The sub-folders and codes in this folder will be lost if it is deleted.");
			var endTime = new Date().getTime();
			if (( endTime - startTime ) < 50 )	// dodgy hack to detect whether the browser blocked the confirm
				r = true;
		}
		if (r)
			inst.delete_node(target_node);
	} else if (lab == "Rename") {
		var newid = 0;
		do {
			newid++;
		} while (inst.get_node("tmpid_"+newid));
		inst.set_id(target_node, "tmpid_"+newid);
		target_node.original.new = "1";
		inst.edit(target_node);
	} else if (lab == "Configure") {
		var link = window.location.href.split('#')[0].replace('code.main','code.edit') + '&index=' + index;
		window.location.replace(link);
	} else if (lab == "Add Code") {
		var new_code_id;
		for (var i=1; ; i++) {
			var test_name = "New Code "+i;
			if( old_code_names.includes(test_name) )
				continue;
			if( (new_code_id = inst.create_node(index, {"text": test_name, "icon": "jstree-file", "folder": "0", "extend": "0", "new": "1"}, "first")) )
				break;
		}
		
		inst.open_node(index);
		inst.edit(new_code_id);
	} else if (lab == "Add Group") {
		var new_group_id;
		for (var i=1; ; i++) {
			var test_name = "New Group "+i;
			if( old_code_names.includes(test_name) )
				continue;
			if( (new_group_id = inst.create_node(index, {"text": test_name, "folder": "1", "extend": "0", "new": "1"}, "first")) )
				break;
		}

		inst.open_node(index);
		inst.edit(new_group_id);
	}
}

function context_func(node, cb_func) {
	// User has requested action pop-up on a node, typically with mouse right-click.
	// We need to call cb_func() with an object of actions applicable to this node type.
	
	var instance = $('#ccccdiv').jstree(true);
	var folder = node.original.folder;
	var path = instance.get_path(node);
	var depth = path.length;

	var actions = {};

	if (node.id == "0") {
		// the only permitted action on root is to add something
		actions.key3 = {"label":"Add Group", "action":action_func }
		actions.key4 = {"label":"Add Code", "action":action_func }
	} else {
		actions.key0 = {"label":"Delete", "action":action_func }
		actions.key1 = {"label":"Rename", "action":action_func }
	
		if (folder != "0") {
			if (depth < 3) {
				actions.key3 = {"label":"Add Group", "action":action_func }
			}
			if (depth < 4) {
				actions.key4 = {"label":"Add Code", "action":action_func }
			}
		} else {
			if ((typeof node.original.new == "undefined") || (node.original.new<1)) {
				actions.key2 = {"label":"Configure", "action":action_func }
			}
		}
	}
	
	cb_func(actions);
}

var move_revert = false;

function check_num_codes(e, data) {
	if ( $('.jstree-node').length > 20 )
		$('#topbuttons').show();
	else
		$('#topbuttons').hide();
}

$(document).ready(function() {

	$('#ccccdiv').on('select_node.jstree', function (e, data) {
		if (typeof data.event == "undefined")
			return;
		if ( (data.event.which == 1) && (data.node.original.folder == "1") ) {
			// for a left-click on a group, toggle its open / closed state
			data.instance.toggle_node(data.node);
		}
	})
	.on('move_node.jstree', function (e, data) {
		if (move_revert) {
			move_revert = false;
			return;
		}
			
		var instance = $(this).jstree(true);
		var parent_node = instance.get_node(data.parent);
		var msg = "";
		
		if (data.parent != '#') {
			// if the new parent is not root, ensure it is a folder
			if (parent_node.original.folder == "0")
				msg = "It is not permitted to place an item inside a code";
			else {
				// check the maximum depth of the whole object tree
				var max_depth = 0;
				max_depth = get_depth(instance,'#',max_depth);
				if (max_depth <= 4) {
					changed = true;
					return;		// move is OK
				}
				
				msg = "Too many levels";
			}
		}
		
		// Error - revert the move
		if(msg != "")
			alert(msg);
		
		// the following move will re-invoke this event handler
		move_revert = true;
		instance.move_node(data.node, data.old_parent, data.old_position);
	})
	.on('ready.jstree', function(e, data) {
		data.instance.open_node('0');
	})
	.on('after_open.jstree', check_num_codes)
	.on('after_close.jstree', check_num_codes)
	.jstree({
		"plugins" : [ "dnd", "unique", "contextmenu", "types" ],
		'core' : {
			'check_callback' : true,
			'data' : {
				'url' : window.location.href.split('#')[0] + '&action=getcccc'
			}
		},
		"contextmenu": { "items": context_func },
		"types": {
			"extend_type" : {
				a_attr:  { "has_extend" : "1" }
			}
		}
	});
	
	$(this).on('dnd_move.vakata.jstree', function(e, data) {
		// this code based on default jstree handler for this event
		if(!data || !data.data || !data.data.jstree) { return; }

		// if we are hovering the marker image do nothing (can happen on "inside" drags)
		if(data.event.target.id && data.event.target.id === 'jstree-marker') { return; }

		var ins = $.jstree.reference(data.event.target), ref = false;
		
		// if we are over an instance ...
		if(ins && ins._data && ins._data.dnd) {
			// if we are hovering the container itself do nothing
			if( (data.event.target === ins.element[0] || data.event.target === ins.get_container_ul()[0]) && ins.get_container_ul().children().length === 0) { return; }

			// if we are hovering a tree node ...
			ref = $(data.event.target).closest('.jstree-anchor');
			if(ref && ref.length && ref.parent().is('.jstree-closed, .jstree-open, .jstree-leaf')) {
				var tnode = ins.get_node(ref);
				if(tnode.original.folder == "0") {
					// not a folder, change green tick to red cross
					data.helper.find('.jstree-icon').first().removeClass('jstree-ok').addClass('jstree-er');
				}
			}
		}
	});

	$('.applycccc').click(function (ev) {
		$('#ccccerr').hide();
		var instance = $('#ccccdiv').jstree(true);
		var flat_array = instance.get_json('#', {flat: true});

		var post_data = "[";
		var i, num = flat_array.length;
		if (num>0) {
			var ok = true;
			for (i=0; i<num; i++) {
				var node = instance.get_node(flat_array[i].id);
				if (node) {
					if (node.id == "0")
						continue;
					var name = node.text;
					if ( /\t/.test(name) || /\s$/.test(name) ) {
						$('#ccccerr').show();
						ok = false;
						instance.deselect_all(true);
						instance.select_node(node, true);
						instance.edit(node);
						break;
					}
					var org = node.original;
					if (org) {
						flat_array[i].extend = org.extend;
						flat_array[i].oldid = org.id;
						if (typeof org.new == "string")
							flat_array[i].new = org.new;
					}
				}
				post_data = post_data + JSON.stringify(flat_array[i]);
				if (i < (num-1))
					post_data = post_data + ',';
			}
			if( !ok )
				return;
				
			post_data += ']';
			$.ajax({
				method: 'POST',
				url: window.location.href.split('#')[0] + '&action=setcccc',
				contentType: 'application/x-json',
				data: post_data,
				success: function (data,textStatus,jqXHR) {
					// response has already been converted from JSON to JS object
					if ((typeof data.result != "undefined") && (data.result == "OK")) {
						changed = false;
						window.location.replace(window.location.href);
					} else {
						alert('Failed to update codes');
						if (typeof data.result != "undefined")
							console.log('cccc.js: '+data.result);
					}
				}
			});
		}
	});
	
	$('.cancelcccc').click(function (ev) {
		var newloc = window.location.href.replace("pagename=code.main","pagename=callreport");
		window.location.replace(newloc);
	});
});
