-- Define new version of SEC_TO_TIME() which can handle values above 3020399 seconds (838:59:59)

DROP FUNCTION IF EXISTS [db].SEC_TO_TIME_F;
	
CREATE FUNCTION [db].SEC_TO_TIME_F (zzz INT(11))
RETURNS VARBINARY(50) DETERMINISTIC
RETURN CONCAT(IF(zzz<36000,'0',''),FLOOR(zzz / 3600),RIGHT(SEC_TO_TIME(zzz % 3600),6));

-- This variant condenses the time value, if possible, to mm:ss

DROP FUNCTION IF EXISTS [db].SEC_TO_TIME_F2;
	
CREATE FUNCTION [db].SEC_TO_TIME_F2 (zzz INT(11))
RETURNS VARBINARY(50) DETERMINISTIC
RETURN IF(zzz<3600,RIGHT(SEC_TO_TIME(zzz),5),CONCAT(FLOOR(zzz / 3600),RIGHT(SEC_TO_TIME(zzz % 3600),6)));


-- NOTE: these functions rely on a specific CallLog JOIN being used when 
--       generating the paramters.
--
-- FROM CallLog
-- LEFT JOIN CallLog AS c2 ON c2.transferConfId = CallLog.confId
-- LEFT JOIN CallLog AS c3 ON c3.confId = CallLog.confId AND c3.transferCall = 4 AND c3.id != CallLog.id (this needed for USE_RING)
-- LEFT JOIN CallLogExtra ON callid = CallLog.id
-- GROUP BY CallLog.id (this is important)

-- To view an existing UDF ...
-- SELECT db,name,body FROM mysql.proc WHERE name='CALL_FILTER';

DROP FUNCTION IF EXISTS [db].CALL_FILTER;

-- This UDF is used by sqllog to filter calls for inclusion in WbCalls. So no need to use again if doing INNER JOIN with WbCalls.

-- Call with MAX(CALL_FILTER (c2.id, CallLog.transfercall, CallClass, CallLog.B_Remote, CallLog.callConnected, CallLog.hasConnected, CallLog.callDuration, CallLog.A_Out, c2.hasConnected, CallLog.A_Name)) AS filter

-- WARNING: using this function in a WHERE expression does not give any errors, but the results are not as desired. Use a sub-query and filter on the function return value.

CREATE FUNCTION [db].CALL_FILTER (c2_id INT(11), c1_transfercall INT(11), CallClass INT(11), B_Remote VARCHAR(255), c1_callConnected DECIMAL(10,3),
  c1_hasConnected INT(11), c1_callDuration DECIMAL(10,3), A_Out INT(11), c2_hasConnected INT(11), A_Name VARCHAR(255))
RETURNS BOOL DETERMINISTIC
RETURN c1_transfercall NOT IN('5') AND NOT(CallClass<=>3)
  AND B_Remote NOT IN ('!LeaveRecording','!AnnouncementRepeat','!AnnouncementInitial','!CollectVoicemail')
  AND IF(c2_id IS NULL, TRUE,
    CASE c1_transfercall
      WHEN 0 THEN c1_callConnected>1
      WHEN 1 THEN c1_hasConnected>0 AND c1_callConnected>1
      WHEN 2 THEN c1_callDuration>2 AND NOT (c1_hasConnected=0 AND c1_callConnected=0 AND A_Out=0 AND c2_hasConnected=1)
      WHEN 3 THEN TRUE
      WHEN 4 THEN TRUE
    END)
  AND NOT (c1_hasconnected=0 AND A_Name LIKE '(%)') AND A_Name != '!WebPhone'
  AND B_Remote NOT LIKE('!Coll%') AND B_Remote NOT LIKE('!Del%');

DROP FUNCTION IF EXISTS [db].CALL_VALID;

-- Call with MAX(CALL_VALID (c2.id, CallLog.transfercall, CallLog.hasConnected, CallLog.callConnected, c2.hasConnected)) AS valid

CREATE FUNCTION [db].CALL_VALID (c2_id INT(11), c1_transfercall INT(11), c1_hasConnected INT(11), c1_callConnected DECIMAL(10,3), c2_hasConnected INT(11))
RETURNS BOOL DETERMINISTIC
RETURN IF(c2_id IS NULL,TRUE,NOT (
  (c1_transfercall=1 AND IF(c1_hasConnected,c1_callConnected<1,c1_callConnected=0) AND c2_hasConnected=1)
  OR
  (c1_transfercall=0 AND c1_hasConnected AND c1_callConnected=0)
));


DROP FUNCTION IF EXISTS [db].USE_RING;

-- Call with MAX(USE_RING(c2.id, CallLog.transfercall, c2.transfercall, CallLog.callConnected, CallLog.callDuration, c2.callConnected, CallLog.B_Remote, c3.id)) AS use_ring

CREATE FUNCTION [db].USE_RING (c2_id INT(11), c1_transfercall INT(11), c2_transfercall INT(11), c1_callConnected DECIMAL(10,3), c1_callDuration DECIMAL(10,3), c2_callConnected DECIMAL(10,3), c1_B_Remote VARCHAR(255), c3_id INT(11))
RETURNS BOOL DETERMINISTIC
RETURN (c1_B_Remote != '!LeaveVoicemail') AND IF(c2_id IS NULL,c3_id IS NULL,NOT (c1_transfercall=0 AND ((c2_transfercall=1 AND c1_callConnected=c1_callDuration AND c2_callConnected>1) OR (c2_transfercall=2 AND c1_callConnected=c1_callDuration))));


DROP FUNCTION IF EXISTS [db].CALL_DURATION;

-- Call with MAX(CALL_DURATION(c2.id, CallLog.callDuration, CallLog.transfercall, CallLog.hasConnected, CallLog.callConnected, c2.transfercall, c2.hasConnected, c2.callConnected, c2.callDuration, CallLog.transferConfid)) AS callDuration

CREATE FUNCTION [db].CALL_DURATION (c2_id INT(11), c1_callDuration DECIMAL(10,3), c1_transfercall INT(11), c1_hasConnected INT(11), c1_callConnected DECIMAL(10,3),
  c2_transfercall INT(11), c2_hasConnected INT(11), c2_callConnected DECIMAL(10,3), c2_callDuration DECIMAL(10,3), c1_xferid VARCHAR(255))
RETURNS DECIMAL(10,3) DETERMINISTIC
RETURN CASE
  WHEN c2_id IS NULL THEN c1_callDuration
  WHEN (c1_transfercall=0 AND c1_hasConnected AND c1_callDuration=c1_callConnected AND c2_transfercall=1)
    OR (c2_transfercall=1 AND c2_hasConnected!=0 AND c2_callConnected<1)
    THEN c2_callDuration-c2_callConnected+c1_callConnected
  WHEN c2_hasConnected=0 AND c2_transfercall=1 AND c1_hasConnected=1 AND c1_transfercall=0 AND c2_callConnected=0 AND c1_xferid=''
    THEN c1_callDuration+c2_callDuration
  ELSE c1_callDuration
END;


DROP FUNCTION IF EXISTS [db].RING_TIME;

-- Call with MAX(RING_TIME(c2.id, CallLog.callDuration, CallLog.transfercall, CallLog.hasConnected, CallLog.callConnected, c2.transfercall, c2.hasConnected, c2.callConnected, c2.callDuration, CallLog.transferConfid)) AS ring_time

CREATE FUNCTION [db].RING_TIME (c2_id INT(11), c1_callDuration DECIMAL(10,3), c1_transfercall INT(11), c1_hasConnected INT(11), c1_callConnected DECIMAL(10,3),
  c2_transfercall INT(11), c2_hasConnected INT(11), c2_callConnected DECIMAL(10,3), c2_callDuration DECIMAL(10,3), c1_xferid VARCHAR(255))
RETURNS DECIMAL(10,3) DETERMINISTIC
RETURN CALL_DURATION (c2_id, c1_callDuration, c1_transfercall, c1_hasConnected, c1_callConnected, c2_transfercall, c2_hasConnected, c2_callConnected, c2_callDuration, c1_xferid) - c1_callConnected;


DROP FUNCTION IF EXISTS [db].ABANDONED;

-- Call with ABANDONED(CallLog.hasConnected, CallLog.transfercall, CallLog.A_Out, c2.hasConnected) AS abandoned

CREATE FUNCTION [db].ABANDONED (c1_hasConnected INT(11), c1_transfercall INT(11), A_Out INT(11), c2_hasConnected INT(11))
RETURNS BOOL DETERMINISTIC
RETURN CASE c1_transfercall
  WHEN 1 THEN (c1_hasConnected + IFNULL(c2_hasConnected,0))=0
  WHEN 2 THEN (c1_hasConnected=0 AND A_Out=1) OR ((c1_hasConnected + IFNULL(c2_hasConnected,0))=0)
  ELSE c1_hasConnected=0
END;

-- The TALK_TIME expression cannot be put in a UDF as it includes aggregating functions.
-- The expression is as follows (using the standard JOIN and GROUP):

-- IF(
--    MAX(c2.confId <=> CallLog.transferConfId),
--    MAX(IF(c2.transfercall=1,IF(c2.confId <=> CallLog.transferConfId,CallLog.callConnected-c2.callDuration,0),CallLog.callConnected)),
--    CallLog.callConnected
--  ) AS talk_time


-- Simpler filter for outbound calls

DROP FUNCTION IF EXISTS [db].CALL_OUT_FILTER;

-- Call with CALL_OUT_FILTER(CallLog.hasConnected, CallLog.callConnected, CallLog.B_Remote, CallLog.transfercall)

CREATE FUNCTION [db].CALL_OUT_FILTER (hasConnected INT(11), callConnected DECIMAL(10,3), B_Remote VARCHAR(255), transfercall INT(11))
RETURNS BOOL DETERMINISTIC
RETURN (hasConnected!=2 OR B_Remote='!LeaveVoicemail')
  AND NOT (transfercall=1 AND hasConnected=0)
  AND NOT (transfercall=0 AND hasConnected AND callConnected=0)
  AND B_Remote NOT IN ('!LeaveRecording','!AnnouncementRepeat','!AnnouncementInitial','!CollectVoicemail')
  AND B_Remote NOT LIKE('!Coll%') AND B_Remote NOT LIKE('!Del%');




